var AlbumView = {
  initiated : false,
  container : null,
  img_container : null,
  caption : null,
  keepFocus : true,
  updatedSelection : false,
  current: 0, target: 0, timer: 0,
  middle : 0, biggest : 0, max : 0,
  MAX_NODES: 6,
  nodeIndexes : new Array(),
  lastOffsets : new Array(),
  itemNodes : new Array(),
  canRefresh : true,
  STEP_INTERVAL : 50,
  init : function(aContainer, aImgContainer, aCaption){
    this.canvas = document.getElementById("albumcanvas");
    this.container = document.getElementById("albumview-container");
    this.img_container = document.getElementById("albumview-image-container");
    this.caption = document.getElementById("albumview-caption");
    this.img_container.addEventListener('click',this.displayClick,false);
    this.img_container.addEventListener('dblclick',this.displayDblClick,false);
    this.container.addEventListener('DOMMouseScroll', this.wheel, false);
    this.container.addEventListener("DOMAttrModified",this.attrModified,false);
    this.container.addEventListener('keypress',this.handleKeypress,true);
    this.initiated = true;
  },
  uninit : function(){
    this.img_container.removeEventListener('click',this.displayClick,false);
    this.img_container.removeEventListener('dblclick',this.displayDblClick,false);
    this.container.removeEventListener('DOMMouseScroll', this.wheel, false);
    this.container.removeEventListener("DOMAttrModified",this.attrModified,true);
    this.container.removeEventListener('keypress',this.handleKeypress,true);
    this.initiated = false;
  },
  attrModified : function(event){
    if (event.target==AlbumView.container &&
        (event.attrName=="height" || event.attrName=="width")){
      //dump("attrmodified: "+AlbumView.canRefresh+"\n");
      if (AlbumView.canRefresh)
        AlbumView.refresh(event);
    }
  },
  /* called on start and on refresh
    caches unchanging values*/
  refresh : function(event) {
    dump("refresh!");
    if (event){
      dump("event: "+event+" attr: "+event.attrName+" values: "+event.prevValue+"|"+event.newValue+"\n");
    }
    var self = AlbumView;
    self.canRefresh = false;
    var images_width = self.container.boxObject.width;//self.img_container.parentNode.parentNode.parentNode.boxObject.width;
    self.middle = images_width * 0.5;
    self.biggest = images_width * 0.3;
    if(self.biggest>300)self.biggest=300;
    images_height = Math.ceil(1.1 * self.biggest - .125 * self.middle);
    //dump("set album size: "+images_width+"\n");
    //self.setSize(self.img_container,images_width,images_height);
    //self.setSize(self.img_container.parentNode.parentNode.parentNode,images_width,images_height+self.caption.boxObject.height);
    //self.img_container.parentNode.parentNode.parentNode.style.maxHeight=images_height+self.caption.boxObject.height+"px";
    //self.setSize(self.canvas,images_width,images_height);
    //self.container.width = images_width;
    self.img_container.width = images_width;
    self.img_container.height = images_height;
    //self.container.setAttribute("width",images_width);
    self.slope = (images_height)/(images_width/2);
    self.height = images_height;
    self.width = self.middle*5/4;
    self.real_width = images_width;
    /* Display images in current order */
    self.moveTo(self.current);
    self.canRefresh = true;
  },
  step : function() {
    var self = AlbumView;
    if (self.target < self.current-1 || self.target > self.current+1)	{
        self.moveTo(self.current + (self.target-self.current)/3);
        setTimeout(self.step, self.STEP_INTERVAL);
        self.timer = 1;
    } else {
      self.timer = 0;
      self.current = self.target;
      var targetIndex = -AlbumView.target/150
      if (AlbumViewUtils && AlbumViewUtils.selectIndex)
        AlbumViewUtils.selectIndex(self.itemNodes[targetIndex]);
    }
  },
  glideToIndex : function(index){
    var target = index * -150;
    AlbumView.glideTo(target);
  },
  glideTo : function(aTarget,updateSelection) {
    /* Animate gliding to new x position */
    var self = AlbumView;
    self.target = aTarget;
    var targetIndex = -self.target/150;
    var currentIndex = -self.current/150;
    if (targetIndex>=self.max){
      if (currentIndex<self.max-1)
        self.target = -(self.max-1)*150;
    }
    if (self.timer == 0 && targetIndex<self.max && targetIndex!=currentIndex){
      setTimeout(self.step, 50);
      self.timer = 1;
    }
    if (targetIndex<self.max){
      /* Display new caption */
      var ele = self.img_container.childNodes[self.nodeIndexes[targetIndex]];
      if (ele.hasAttribute("caption"))
        self.caption.textContent=ele.getAttribute("caption");
        //self.caption.firstChild.nodeValue = ele.getAttribute("caption");
    }
  },
  /* basically adjusts the zIndex in the stack
    It should be made more efficient later */
  shiftImage : function(index, newpos, image){
    var nodeIndex = AlbumView.nodeIndexes[index];
    if (nodeIndex!=newpos){
      image = AlbumView.img_container.insertBefore(image,AlbumView.img_container.childNodes[newpos]);
      var children = AlbumView.img_container.childNodes;
      for (var i = 0; i<= AlbumView.max - 1;i++){
        AlbumView.nodeIndexes[children[i].value] = i;
      }
    }
  },
  /* main part of the transition */
  moveTo : function(x) {
    try{
    var self = AlbumView;
    self.current = x;
    //dump("container: "+self.container.id+"|"+self.container.firstChild.id+"\n");
    //var contLeft = self.container.firstChild.boxObject.x; contLeft = 0;
    var contLeft = 0;
    //dump("contLeft: "+contLeft+"\n");
    var zIndex = self.end;
    var targetIndex = -self.target / 150;
    var currentIndex = -self.current / 150;
    var direction = self.lastOffsets[0] + targetIndex;
    //if (direction<0){
    //  dump(direction+" add to bottom\n");
    //}
    //else if (direction>0){
    //  dump(direction+" remove from bottom\n");
    //}

    var start=targetIndex-self.MAX_NODES;
    if (start<0) start=0;
    var end=targetIndex+self.MAX_NODES;
    if (end>=self.max) end = self.max - 1;
    function moveLoop(index,increasing){
      var nodeIndex = self.nodeIndexes[index];
      var image = self.img_container.childNodes.item(nodeIndex);
      var hide = index<start || index>end;
      if (hide){
        image.hidden = true;
      }
      else{
        if (image.hidden) image.hidden = false;
        /* Get current image properties */
        var offset = index - targetIndex;//targetIndex;
        var lastOffset = self.lastOffsets[index];
        var absOffset = Math.abs(offset);
        var left,top,imgwidth,imgheight;

        var z = Math.sqrt(10000 + Math.pow(x,2))+100;
        imgwidth = 200 / z * self.biggest;
        left = (x / (Math.sqrt(10000 + Math.pow(x,2))+300))*self.width + self.middle - imgwidth/2;
        top = 25 / (Math.sqrt(10000 + Math.pow(x,2))+100) * self.width;
        imgheight = 220 / (Math.sqrt(10000 + Math.pow(x,2))+100) * self.biggest - top
        top = (self.height - imgheight)/2;

        if (left<0 || left+imgwidth>self.real_width){
          image.hidden=true;
        }

        /* Set new image properties */
        //dump(Math.ceil(left) + "|" + contLeft+"\n");
        image.setAttribute("left", Math.ceil(left) - contLeft);
        image.setAttribute("top", parseInt(top));
        self.setSize(image.firstChild,imgwidth,imgheight);
        self.setSize(image,imgwidth,imgheight);

        self.lastOffsets[index] = offset;
      }
      if (increasing) x += 150; else x -= 150;
    }
    /* Loop */
    //for (var index = 0; index < currentIndex; index++)

    for (var index = 0; index < self.max; index++)
      moveLoop(index,true);
    //x = current + max * 150 - 150;
    //for (var index = max - 1; index>=currentIndex; index--)
    //  moveLoop(index, false);
    for (var index = 0; index < self.max; index++){
      var offset = index - targetIndex;
      var nodeIndex = self.nodeIndexes[index];
      var image = self.img_container.childNodes.item(nodeIndex);
      var expected = offset<=0?self.max-1+offset:self.max - 1 - targetIndex - offset;
      //dump(index+") "+expected+"\n");
      self.shiftImage(index,expected,image);
    }
    }catch(e){Components.utils.reportError("MOVETO: "+e);}
  },
  displayClick : function(event){
    if (event.target.tagName=="image" || event.target.tagName=="canvas"){
      if (event.button == 1)
        AlbumViewUtils.performAction(AlbumView.itemNodes[event.target.parentNode.value],event);
      else
        AlbumView.glideTo(event.target.parentNode.value * -150,true);
    }
  },
  displayDblClick : function(event){
    if (event.target.tagName=="image" || event.target.tagName=="canvas"){
      AlbumViewUtils.performAction(AlbumView.itemNodes[event.target.parentNode.value],event);
    }
  },
  /* JavaScript mouse wheel & keyboard support */
  handle : function(delta) {
    if  (delta > 0) {
      if(AlbumView.target != 0)
        AlbumView.target += 150;
    } else if(-AlbumView.current/150 < AlbumView.max) {
        AlbumView.target = AlbumView.target - 150;
    }
    AlbumView.glideTo(AlbumView.target,true);
  },
  /* JavaScript mouse wheel support */
  wheel : function(event){
    var delta = -event.detail/3;
    AlbumView.handle(delta);
    event.preventDefault();
  },
  handleKeypress : function(event){
    var KeyEvent = Ci.nsIDOMKeyEvent;
    switch(event.keyCode){
      case KeyEvent.DOM_VK_LEFT:
        AlbumView.handle(1)
        break;
      case KeyEvent.DOM_VK_RIGHT:
        AlbumView.handle(-1);
        break;
      //TODO: This won't work since it isn't focusable anymore
      case KeyEvent.DOM_VK_RETURN:
      case KeyEvent.DOM_VK_ENTER:
        if (AlbumViewUtils && AlbumViewUtils.performAction){
          var targetIndex = -AlbumView.target/150;
          AlbumViewUtils.performAction(AlbumView.itemNodes[targetIndex]);
        }
    }
  },
  createImages : function(aImages){
    try{
    var totalImages = aImages;
    AlbumView.max = totalImages.length;
    AlbumView.nodeIndexes = new Array(AlbumView.max);
    AlbumView.itemNodes = new Array(AlbumView.max);
    /* remove any remaining children */
    while (AlbumView.img_container.lastChild)
      AlbumView.img_container.removeChild(AlbumView.img_container.lastChild);
    for (var i = AlbumView.max-1; i>=0; i--){
      var src = totalImages[i];
      AlbumView.nodeIndexes[i]=AlbumView.max-i-1;
      AlbumView.lastOffsets[i]=i;
      AlbumView.itemNodes[i]=src.node;
      var box = document.createElement("vbox");
      box.setAttribute("class","album");
      var imageEle = document.createElement("image");
      imageEle.setAttribute("src",src.image);
      box.appendChild(imageEle);
      //var line = document.createElement("hbox");
      //line.setAttribute("style","background-color: green;");
      //AlbumView.setSize(line,4,10);
      //var hbox = document.createElement("hbox");
      //hbox.setAttribute("align","center");
      //hbox.setAttribute("pack","center");
      //hbox.appendChild(line);
      //box.appendChild(hbox);
      box.value = i;
      if ("caption" in src){
        box.setAttribute("caption",src.caption);
      }
      else{
        box.setAttribute("caption","No Caption");
      }
      //if (i-AlbumView.MAX_NODES>0)
      //  box.setAttribute("hidden","true");
      AlbumView.img_container.appendChild(box);
      box.width = 200; box.height = 170;
      //AlbumView.setSize(box,"200","170");

    }
    }catch(e){Components.utils.reportError(e);}
  },
  setSize : function(ele,width,height){
    ele.setAttribute("height",height);
    ele.setAttribute("width",width);
    //ele.width=width;
    //ele.height=height;
    //ele.style.maxWidth = width+"px";
    //ele.style.maxHeight = height+"px";
    //ele.setAttribute("style","max-height: "+height+"px; max-width: "+width+"px");
  }
}
