dump("preview service\n");
var PreviewService = {
  PREVIEW_WIDTH: 300,
  timeouts: [],
  bookmarks: null,
  onLoad: function() {
    gBrowser.addEventListener("load", PreviewService.onPageLoad, true);
    var appInfo = Components.classes["@mozilla.org/xre/app-info;1"]
                        .getService(Components.interfaces.nsIXULAppInfo);
    var versionChecker = Components.classes["@mozilla.org/xpcom/version-comparator;1"]
                                   .getService(Components.interfaces.nsIVersionComparator);
    var FF3 = (versionChecker.compare(appInfo.version, "3.0a") >= 0);
    if (FF3)
      PreviewService.bookmarks = new PreviewServiceFF3();
    else
      PreviewService.bookmarks = new PreviewServiceFF2();
  },
  onUnload: function() {
    gBrowser.removeEventListener("load", PreviewService.onPageLoad, true);
    PreviewService.bookmarks.unload();
  },
  onPageLoad: function(event){
    if (event.originalTarget instanceof HTMLDocument) {
      var doc = event.originalTarget;
      if (event.originalTarget.defaultView.frameElement) {
        // Frame within a tab was loaded. doc should be the root document of
        // the frameset.
        // Find the root document:
        while (doc.defaultView.frameElement) {
          doc=doc.defaultView.frameElement.ownerDocument;
        }
      }
      var uri = makeURI(doc.URL);
      if (uri && PreviewService.bookmarks.isBookmarked(uri)){
        try{
        if (PreviewService.timeouts[doc.URL]!=null)
          clearTimeout(PreviewService.timeouts[doc.URL]);
        }catch(e){dump(e+"\n");}
        PreviewService.timeouts[doc.URL]=setTimeout(function(){
          /* Just to give it time to load a little more... */
          //dump("timeout: "+PreviewService.timeouts+"\n");
          PreviewService.timeouts[doc.URL]=null;
          PreviewService.createPreviewFromDoc(doc);
        },1500);
      }
    }
  },
  bookmarkAdded : function(aURL,aArg){
    var numTabs = gBrowser.tabContainer.childNodes.length;
    for(var index=0; index<numTabs; index++) {
      var currentBrowser = gBrowser.getBrowserAtIndex(index);
      if (aURL == currentBrowser.currentURI.spec) {
        dump("matched!: "+index+"\n");
        PreviewService.createPreviewFromBrowser(currentBrowser, aArg);
        break;
      }
    }
  },
  createPreviewFromDoc: function(doc) {
    var browser=gBrowser.getBrowserForDocument(doc);
    if (browser)
      this.createPreviewFromBrowser(browser);
  },
  createPreviewFromBrowser : function(browser, aArg){
    dump("creatpreviewfrombrowser: "+browser.currentURI.spec+"\n");
    var win=browser.contentWindow;
    var canvas = document.createElementNS("http://www.w3.org/1999/xhtml", "canvas");
    var scale=this.PREVIEW_WIDTH/(win.innerWidth);
    canvas.height = Math.floor(scale*(win.innerHeight));
    canvas.width = this.PREVIEW_WIDTH;
    var ctx = canvas.getContext("2d");
    ctx.save();
    ctx.scale(scale, scale);
    ctx.drawWindow(win, 0, 0, win.innerWidth,win.innerHeight,"rgb(255,255,255)");
    ctx.restore();
    var data = canvas.toDataURL("image/jpeg", "");
    if(data && data != "") {
      try {
        PreviewService.bookmarks.savePreview(data,browser.currentURI,aArg);
      } catch(e) {
        dump("Error saving history image for uri\n"+browser.currentURI.spec+"\nError: "+e+"\n");
      }
    }
  },
  generatePreview : function(){
    if (!this._index && this._index!=0) {
      this._index=-1;
      this._createIframe();
    }
    this._index++;
    if (this.all && this._index<this.all.length){
      var url = PreviewService.bookmarks.getBookmarkURL(this.all[this._index]);
      if (url && !PreviewService.bookmarks.hasPreview(url)){
        this._iframe.loadURI(url,null,null);
        this._timeoutID = setTimeout(function(){
          delete this._timeoutID;
          PreviewService.generatePreview();
        },60000);
      }
      else
        this.generatePreview();
    }
    else{
      this._iframe.removeEventListener("load", this._iframeLoaded, true);
      //this._iframe.removeEventListener("DOMContentLoaded", this._iframeLoaded, true);
      //this._iframe.parentNode.removeChild(this._iframe);
      delete this._iframe;
      delete this._index;
      delete this.all;
    }
  },
  generateAllPreviews : function(){
    this.all = PreviewService.bookmarks.getAllBookmarks();
    this.generatePreview();
  },
  _createIframe : function(){
    gBrowser.selectedTab = gBrowser.addTab("about:blank");
    this._iframe = gBrowser.getBrowserForTab(gBrowser.selectedTab);
    this._iframe.addEventListener("load", this._iframeLoaded, true);
  },
  _iframeLoaded : function(){
    dump("iframe loaded\n");
    clearTimeout(PreviewService._timeoutID);
    delete PreviewService._timeoutID;
    setTimeout(function(){
      PreviewService.createPreviewFromBrowser(PreviewService._iframe);
      PreviewService.generatePreview();
    },4000);
  }
}
function PreviewServiceFF3(){
  PlacesUtils.bookmarks.addObserver(this, false);
  this.dir = Components.classes["@mozilla.org/file/directory_service;1"]
                     .getService(Components.interfaces.nsIProperties)
                     .get("ProfD", Components.interfaces.nsIFile);
  this.dir.append("bookmarkPreviews");
  if (!this.dir.exists())
    this.dir.create(Components.interfaces.nsIFile.DIRECTORY_TYPE,0777);
}
PreviewServiceFF3.prototype = {
  isBookmarked : function(uri){
    return PlacesUtils.bookmarks.isBookmarked(uri);
  },
  getAllBookmarks : function(){

  },
  hasPreview : function(aUrl){
    return this._getPreviewFile(aUrl).exists();
  },
  _getPreviewFile : function(aURL){
    var file = this.dir.clone();
    file.append(encodeURIComponent(aURL.toLowerCase())+".jpg");
    return file;
  },
  savePreview : function(data,uri,aArg){
    var file = this._getPreviewFile(uri.spec);
    var io = Components.classes["@mozilla.org/network/io-service;1"]
                       .getService(Components.interfaces.nsIIOService);
    var source = io.newURI(data, "UTF8", null);
    var target = io.newFileURI(file)
    // prepare to save the data
    var persist = Components.classes["@mozilla.org/embedding/browser/nsWebBrowserPersist;1"]
                            .createInstance(Components.interfaces.nsIWebBrowserPersist);
    persist.persistFlags = Components.interfaces.nsIWebBrowserPersist.PERSIST_FLAGS_REPLACE_EXISTING_FILES;
    persist.persistFlags |= Components.interfaces.nsIWebBrowserPersist.PERSIST_FLAGS_AUTODETECT_APPLY_CONVERSION;

    persist.saveURI(source, null, null, null, null, file);
  },
  unload : function(){
    PlacesUtils.bookmarks.removeObserver(this,false);
  },
  /* nsINavBookmarkObserver */
  onBeginUpdateBatch : function(){},
  onEndUpdateBatch : function(){},
  onItemAdded : function(aItemId, aFolder, aIndex){
    var uri = PlacesUtils.bookmarks.getBookmarkURI(aItemId);
    PreviewService.bookmarkAdded(uri.spec,aItemId);
  },
  onItemRemoved : function(aItemId,  aFolder,  aIndex){
    //PlacesUtils.annotations.removeItemAnnotation(aItemId,"bookmark/preview");
    var file = this._getPreviewFile(PlacesUtils.bookmarks.getBookmarkURI(aItemId).spec);
    if (file.exists())
      file.remove(false);
  },
  onItemChanged : function(aBookmarkId, aProperty, aIsAnnotationProperty, aValue){},
  onItemVisited : function(aBookmarkId,  aVisitID, time){},
  onItemMoved : function(aItemId,  aOldParent,  aOldIndex,  aNewParent,  aNewIndex){},
  createInstance: function (outer, iid) {
    if (outer != null)
      throw Components.results.NS_ERROR_NO_AGGREGATION;
    return this.QueryInterface(iid);
  },

  QueryInterface: function PS_QueryInterface(aIID) {
    if (aIID.equals(Ci.nsINavBookmarkObserver) ||
         aIID.equals(Ci.nsISupports))
      return this;
    throw Cr.NS_NOINTERFACE;
  }
}
function PreviewServiceFF2(){
  //this.bmsvc = Components.classes["@mozilla.org/browser/bookmarks-service;1"].
  //                  getService(Components.interfaces.nsIBookmarksService);
  initServices();
  initBMService();
  this.bmsvc = BMSVC;
  var bkmkTxnSvc = Components.classes["@mozilla.org/bookmarks/transactionmanager;1"]
            .getService(Components.interfaces.nsIBookmarkTransactionManager);
  bkmkTxnSvc.transactionManager.AddListener(this);

  this.dir = Components.classes["@mozilla.org/file/directory_service;1"]
                     .getService(Components.interfaces.nsIProperties)
                     .get("ProfD", Components.interfaces.nsIFile);
  this.dir.append("bookmarkPreviews");
  if (!this.dir.exists())
    this.dir.create(Components.interfaces.nsIFile.DIRECTORY_TYPE,0777);
}
PreviewServiceFF2.prototype = {
  unload : function(){
    var bkmkTxnSvc = Components.classes["@mozilla.org/bookmarks/transactionmanager;1"]
            .getService(Components.interfaces.nsIBookmarkTransactionManager);
    bkmkTxnSvc.transactionManager.RemoveListener(this);
  },
  isBookmarked : function(uri){
    var NC = "http://home.netscape.com/NC-rdf#";
    var RDF = Components.classes["@mozilla.org/rdf/rdf-service;1"].getService(Components.interfaces.nsIRDFService);
   	var BMDS  = RDF.GetDataSource("rdf:bookmarks");
    var urlArc = RDF.GetResource(NC+"URL");
    var elements = BMDS.GetAllResources();
    while (elements.hasMoreElements()){
      var ele = elements.getNext();
      if(BMDS.hasArcOut(ele, urlArc)){
        var target = BMDS.GetTarget(ele, urlArc, true);
        var value = target.QueryInterface(Components.interfaces.nsIRDFLiteral).Value;
        //dump("val: "+value+"|"+uri.spec+"\n");
        if(value == uri.spec){
            dump("isBookmark - true - "+uri.spec+"\n");
            return true
        }
      }
    }
    dump("isBookmark - false - "+uri.spec+"\n");
    return false;
  },
  hasPreview : function(aUrl){
    return this._getPreviewFile(aUrl).exists();
  },
  _getPreviewFile : function(aURL){
    var file = this.dir.clone();
    file.append(encodeURIComponent(aURL.toLowerCase())+".jpg");
    return file;
  },
  savePreview : function(data,uri,aArg){
    var file = this._getPreviewFile(uri.spec);
    var io = Components.classes["@mozilla.org/network/io-service;1"]
                       .getService(Components.interfaces.nsIIOService);
    var source = io.newURI(data, "UTF8", null);
    var target = io.newFileURI(file)
    // prepare to save the data
    var persist = Components.classes["@mozilla.org/embedding/browser/nsWebBrowserPersist;1"]
                            .createInstance(Components.interfaces.nsIWebBrowserPersist);
    persist.persistFlags = Components.interfaces.nsIWebBrowserPersist.PERSIST_FLAGS_REPLACE_EXISTING_FILES;
    persist.persistFlags |= Components.interfaces.nsIWebBrowserPersist.PERSIST_FLAGS_AUTODETECT_APPLY_CONVERSION;

    persist.saveURI(source, null, null, null, null, file);
  },
  removePreview : function(url){
    var file = this._getPreviewFile(url);
    file.remove(false);
  },
  observeTransaction : function(aTxn, aDo){
    aTxn = aTxn.wrappedJSObject;
    var type = aTxn.type;
    dump("action type: "+type+"|"+aDo+"\n");
    var item = aTxn.item;
    var itemType = BookmarksUtils.resolveType(item);
    var itemURL = null;
    if (itemType == "Bookmark"){
      var urlProp = RDF.GetResource("http://home.netscape.com/NC-rdf#URL");
      item.QueryInterface(kRDFRSCIID);
      itemURL = BMDS.GetTarget(item, urlProp, true);
      if (itemURL){
        itemURL = itemURL.QueryInterface(kRDFLITIID).Value;
        if ((type == "insert" && aDo) || (type == "remove" && !aDo)){
          PreviewService.bookmarkAdded(itemURL);
        }
        else if ((type == "remove" && aDo) || (type == "insert" && !aDo)){
          this.removePreview(itemURL);
        }
      }
    }
  },
  /* Transaction Listener */
  willDo: function (aTxmgr, aTxn) {},
  didDo : function (aTxmgr, aTxn) {
    this.observeTransaction(aTxn, true);
  },
  willUndo: function (aTxmgr, aTxn) {},
  didUndo : function (aTxmgr, aTxn) {
    this.observeTransaction(aTxn, false);
  },
  willRedo: function (aTxmgr, aTxn) {},
  didRedo : function (aTxmgr, aTxn) {
    this.observeTransaction(aTxn, true);
  },
  didMerge       : function (aTxmgr, aTxn) {},
  didBeginBatch  : function (aTxmgr, aTxn) {},
  didEndBatch    : function (aTxmgr, aTxn) {
    this.observeTransaction(aTxn, this.mLastTxnWasDo);
  },
  willMerge      : function (aTxmgr, aTxn) {},
  willBeginBatch : function (aTxmgr, aTxn) {},
  willEndBatch   : function (aTxmgr, aTxn) {},

  /* Used for generating all previews... not finished */
  getAllBookmarks : function(){
    var all = [];
    BookmarksUtils.getAllChildren(RDF.GetResource("NC:BookmarksRoot"),all);
    return all;
  },
  getBookmarkURL : function(bkmk){
    /*
    [Name,URL,ShortcutURL,Description,WebPanel,FeedURL,
    MicsumGenURI,MicsumExpiration,GeneratedTitle];
    */
    var url = bkmk[1];
    if (url)
      return url.Value;
    else
      return null;
  },
}
window.addEventListener('load',function(e) { PreviewService.onLoad(e); },false);
window.addEventListener('unload',function(e) { PreviewService.onUnload(e); },false);
